@@ -247,7 +247,11 @@ gpg_error_t Pwmd::open(bool force)
     commandMutex->lock();
 
     if (!rc) {
+#if LIBPWMD_VERSION >= 0x603
+	rc = pwmd_setopt(pwm, PWMD_OPTION_LOCK_ON_OPEN, lockOnOpen());
+#else
 	rc = pwmd_command(pwm, NULL, "SET LOCK_ON_OPEN=%i", lockOnOpen());
+#endif
 
 	if (!rc)
 	    rc = pwmd_command(pwm, NULL, "SET RC_ON_LOCKED=%i", errorOnLocked());
