@@ -60,6 +60,9 @@ PwmdDialog::PwmdDialog(const QString &s, const QString &f, const QString &name,
     ui.le_identity->setText(cfg.value("identity", "").toString());
     ui.le_knownhosts->setText(cfg.value("knownhosts", "").toString());
     ui.ck_remote->setChecked(cfg.value("remote", false).toBool());
+    ui.sp_clipboardTimeout->setValue(cfg.value("clipboardTimeout", 20).toInt());
+    clipboardTimer = new QTimer(this);
+    connect(clipboardTimer, SIGNAL(timeout()), SLOT(slotClearClipboard()));
 
     if (ui.le_knownhosts->text().isEmpty())
 	ui.le_knownhosts->setText(QDir::home().path() + "/.ssh/known_hosts");
@@ -196,6 +199,7 @@ PwmdDialog::PwmdDialog(const QString &s, const QString &f, const QString &name,
     connect(ui.tb_hilightColor, SIGNAL(clicked()), this, SLOT(slotChooseHilightColor()));
     connect(ui.tb_clearCache, SIGNAL(clicked()), this, SLOT(slotClearCacheEntry()));
     connect(ui.le_password, SIGNAL(textEdited(const QString &)), this, SLOT(slotPasswordContentChanged(const QString &)));
+    ui.onlyLineEdit->setValidator(new PwmdPasswordOnlyValidate());
     connect(ui.ck_showPassword, SIGNAL(clicked()), this, SLOT(slotShowPassword()));
     connect(ui.tb_generate, SIGNAL(clicked()), this, SLOT(slotGeneratePassword()));
     connect(ui.generateComboBox, SIGNAL(activated(int)), this, SLOT(slotGenerateTypeChanged(int)));
@@ -269,6 +273,26 @@ PwmdDialog::~PwmdDialog()
     cfg.setValue("identity", ui.le_identity->text());
     cfg.setValue("knownhosts", ui.le_knownhosts->text());
     cfg.setValue("ipVersion", ipVersion);
+    cfg.setValue("clipboardTimeout", ui.sp_clipboardTimeout->value());
+}
+
+void PwmdDialog::slotClearClipboard()
+{
+    clearClipboard();
+}
+
+void PwmdDialog::clearClipboard()
+{
+    QClipboard *c = QApplication::clipboard();
+
+    c->setText("", QClipboard::Selection);
+    c->setText("");
+}
+
+void PwmdDialog::closeEvent(QCloseEvent *ev)
+{
+    (void)ev;
+    doFinished();
 }
 
 void PwmdDialog::slotIpVersionToggled(bool b)
@@ -612,6 +636,9 @@ void PwmdDialog::slotClipboardPassword()
 
     c->setText(ui.le_password->text(), QClipboard::Selection);
     c->setText(ui.le_password->text());
+
+    if (ui.sp_clipboardTimeout->value())
+	clipboardTimer->start(ui.sp_clipboardTimeout->value()*1000);
 }
 
 void PwmdDialog::slotAttributesOk()
@@ -745,10 +772,20 @@ void PwmdDialog::slotOptionsOk()
     bool mod = false;
 
     if (state != Pwmd::Init) {
-	gpg_error_t rc;
-
 	if (iterationsOrig != (unsigned)ui.sp_iterations->value()) {
+#if LIBPWMD_VERSION >= 0x603
+	    pwmd_setopt(pwm->handle(), PWMD_OPTION_ITERATIONS,
+		    ui.sp_iterations->value());
+#else
+	    gpg_error_t rc;
+
 	    pwm->command(QString("SET iterations=%1").arg(ui.sp_iterations->value()), rc);
+
+	    if (rc) {
+		showError(rc);
+		return;
+	    }
+#endif
 	    mod = true;
 	}
     }
@@ -913,14 +950,15 @@ void PwmdDialog::slotRefreshFilenameTree()
 bool PwmdDialog::doFinished(bool done)
 {
     if (!ui.pb_update->isEnabled() &&
-	!ui.buttonBox->button(QDialogButtonBox::Apply)->isEnabled()) {
+	!ui.buttonBox->button(QDialogButtonBox::Apply)->isEnabled() &&
+	!ui.tb_passwordUpdate->isEnabled()) {
 
 	if (done)
 	    accept();
 
 	return true;
     }
-    else if (ui.pb_update->isEnabled()) {
+    else if (ui.pb_update->isEnabled() || ui.tb_passwordUpdate->isEnabled()) {
 	gpg_error_t rc = doSaveContent();
 
 	if (rc) {
@@ -1109,6 +1147,10 @@ gpg_error_t PwmdDialog::doSaveContent()
 	ui.tb_passwordUpdate->setEnabled(false);
 	ui.textEdit->document()->setModified(false);
 	ui.le_password->setModified(false);
+
+	if (ui.stackedWidgetRight->currentWidget() == ui.passwordContentPage)
+	    ui.lengthSpinBox->setValue(ui.le_password->text().length());
+
 	setModified();
     }
 
@@ -1435,6 +1477,7 @@ bool PwmdDialog::elementSelected(QTreeWidgetItem *item)
 		ui.stackedWidgetRight->setCurrentWidget(ui.passwordContentPage);
 
 	    setPasswordContent(s);
+	    ui.lengthSpinBox->setValue(s.isEmpty() ? 16 : s.length());
 	    return true;
 	}
 	else if (rc) {
@@ -2268,28 +2311,36 @@ void PwmdDialog::slotGeneratePassword()
 {
     QString result = QString::null;
 
-    switch (ui.generateComboBox->currentIndex()) {
-	case 0:
-	    result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Any, ui.lengthSpinBox->value());
-	    break;
-	case 1:
-	    result = PwmdGenPassDialog::generate(PwmdGenPassDialog::AlphaNumeric, ui.lengthSpinBox->value());
-	    break;
-	case 2:
-	    result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Alpha, ui.lengthSpinBox->value());
-	    break;
-	case 3:
-	    result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Numeric, ui.lengthSpinBox->value());
-	    break;
-	case 4:
-	    if (ui.onlyLineEdit->text().isEmpty())
+    do {
+	switch (ui.generateComboBox->currentIndex()) {
+	    case 0:
+		result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Any,
+			ui.lengthSpinBox->value());
 		break;
+	    case 1:
+		result = PwmdGenPassDialog::generate(PwmdGenPassDialog::AlphaNumeric,
+			ui.lengthSpinBox->value());
+		break;
+	    case 2:
+		result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Alpha,
+			ui.lengthSpinBox->value());
+		break;
+	    case 3:
+		result = PwmdGenPassDialog::generate(PwmdGenPassDialog::Numeric,
+		       	ui.lengthSpinBox->value());
+		break;
+	    case 4:
+		if (ui.onlyLineEdit->text().isEmpty())
+		    break;
 
-	    result = PwmdGenPassDialog::generate(ui.onlyLineEdit->text(), ui.lengthSpinBox->value());
-	    break;
-	default:
-	    break;
-    }
+		result = PwmdGenPassDialog::generate(ui.onlyLineEdit->text(),
+			ui.lengthSpinBox->value());
+		break;
+	    default:
+		break;
+	}
+    } while (!QString::compare(result, ui.le_password->text()) &&
+	    ui.onlyLineEdit->text().length() > 1);
 
     if (result.isEmpty())
 	return;
