@@ -28,6 +28,7 @@ PwmdEdit::PwmdEdit(int argc, char **argv) : QApplication(argc, argv)
     filename = QString::null;
     socket = QString::null;
     QStringList args = arguments();
+    bool editOnly = false;
 
     for (int i = 1; i < args.count(); i++) {
 	QString arg = args.at(i);
@@ -37,20 +38,31 @@ PwmdEdit::PwmdEdit(int argc, char **argv) : QApplication(argc, argv)
 		case 's':
 		    socket = args.at(++i);
 		    break;
+		case 'e':
+		    editOnly = true;
+		    break;
 	    }
 	}
 	else
 	    filename = arg;
     }
 
-    setQuitOnLastWindowClosed(false);
-    shortcutMenu = new QMenu();
-    connect(shortcutMenu, SIGNAL(triggered(QAction *)), this, SLOT(slotShortCut(QAction *)));
-    refreshShortcuts();
-    sti = new QSystemTrayIcon(QIcon(":trayicon.svg"), this);
-    connect(sti, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(slotSystemTrayActivated(QSystemTrayIcon::ActivationReason)));
-    sti->setContextMenu(shortcutMenu);
-    sti->show();
+    if (!editOnly) {
+	clipboardTimer = new QTimer(this);
+	connect(clipboardTimer, SIGNAL(timeout()), SLOT(slotClearClipboard()));
+	setQuitOnLastWindowClosed(false);
+	shortcutMenu = new QMenu();
+	connect(shortcutMenu, SIGNAL(triggered(QAction *)), this, SLOT(slotShortCut(QAction *)));
+	refreshShortcuts();
+	sti = new QSystemTrayIcon(QIcon(":trayicon.svg"), this);
+	connect(sti, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), SLOT(slotSystemTrayActivated(QSystemTrayIcon::ActivationReason)));
+	sti->setContextMenu(shortcutMenu);
+	sti->show();
+    }
+    else {
+	slotEditPwmd();
+	QTimer::singleShot(1, this, SLOT(quit()));
+    }
 }
 
 void PwmdEdit::refreshShortcuts()
@@ -87,10 +99,7 @@ void PwmdEdit::refreshShortcuts()
 
 void PwmdEdit::slotClearClipboard()
 {
-    QClipboard *c = QApplication::clipboard();
-
-    c->setText("", QClipboard::Selection);
-    c->setText("");
+    PwmdDialog::clearClipboard();
 }
 
 void PwmdEdit::slotEditShortcuts()
@@ -127,9 +136,10 @@ void PwmdEdit::slotShortCut(QAction *a)
 
 	c->setText(result, QClipboard::Selection);
 	c->setText(result);
+	refreshShortcuts();
 
 	if (clipboardTimeout)
-	    QTimer::singleShot(clipboardTimeout*1000, this, SLOT(slotClearClipboard()));
+	    clipboardTimer->start(clipboardTimeout*1000);
     }
 }
 
